@@ -1,6 +1,6 @@
 """Provides the main() routine and ColaApplication"""
 # pylint: disable=unused-import
-from __future__ import division, absolute_import, unicode_literals
+from __future__ import absolute_import, division, print_function, unicode_literals
 from functools import partial
 import argparse
 import os
@@ -100,7 +100,7 @@ def setup_environment():
     elif ssh_askpass:
         askpass = ssh_askpass
     elif sys.platform == 'darwin':
-        askpass = resources.share('bin', 'ssh-askpass-darwin')
+        askpass = resources.package_command('ssh-askpass-darwin')
     else:
         askpass = '/usr/X11R6/bin/ssh-askpass'
 
@@ -514,7 +514,7 @@ def async_update(context):
 
     """
     update_status = partial(context.model.update_status, update_index=True)
-    task = qtutils.SimpleTask(context.view, update_status)
+    task = qtutils.SimpleTask(update_status)
     context.runtask.start(task)
 
 
@@ -593,6 +593,7 @@ class ApplicationContext(object):
         self.selection = None  # selection.SelectionModel
         self.fsmonitor = None  # fsmonitor
         self.view = None  # QWidget
+        self.browser_windows = []  # list of browse.Browser
 
     def set_view(self, view):
         """Initialize view-specific members"""
